package tp8;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import tp8.fenetres.FenetreBoutons;
import tp8.fenetres.FenetreCommentFaire;
import tp8.fenetres.FenetreConfig;
import tp8.fenetres.FenetreConversion;
import tp8.fenetres.FenetreDiaporama;
import tp8.fenetres.FenetreSuggestion;
import tp8.fenetres.FenetreTexte;

/**
 *
 * @author brunomermet
 */
public class Application extends JFrame {

	private JInternalFrame conversion;
	private JInternalFrame texte;
	private JInternalFrame diaporama;
	private JInternalFrame boutons;
	private JInternalFrame commentFaire;
	private JInternalFrame config;
	private Action actionAfficherConversion;
	private Action actionAfficherTexte;
	private Action actionAfficherDiaporama;
	private Action actionAfficherBoutons;
	private Action actionAfficherCommentFaire;
	private Action actionAfficherConfig;

	private static JMenuItem itemConversion, itemTexte, itemDiaporama, itemBoutons;

	public Application(String utilisateur) {
		super("multi-fenêtres");
		this.setContentPane(new JDesktopPane());

		Application.ajouterUtilisateur(utilisateur);

		// ****** Barre de menu ******
		JMenuBar barre = new JMenuBar();
		// ------ menu Fichier ------
		JMenu menuFichier = new JMenu("Fichier");
		menuFichier.setMnemonic(KeyEvent.VK_F);
		JMenuItem quitter = new JMenuItem("Quitter");
		quitter.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent aev) {
				System.exit(0);
			}
		});
		quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
		menuFichier.add(quitter);
		barre.add(menuFichier);
		this.setJMenuBar(barre);
		// ------ menu Applications ------
		JMenu menuApplication = new JMenu("Applications");
		menuApplication.setMnemonic(KeyEvent.VK_A);
		actionAfficherConversion = new ActionAfficherConversion();
		itemConversion = new JMenuItem(actionAfficherConversion);
		menuApplication.add(itemConversion);
		actionAfficherTexte = new ActionAfficherTexte();
		itemTexte = new JMenuItem(actionAfficherTexte);
		menuApplication.add(itemTexte);
		actionAfficherDiaporama = new ActionAfficherDiaporama();
		itemDiaporama = new JMenuItem(actionAfficherDiaporama);
		menuApplication.add(itemDiaporama);
		actionAfficherBoutons = new ActionAfficherBoutons();
		itemBoutons = new JMenuItem(actionAfficherBoutons);
		menuApplication.add(itemBoutons);
		barre.add(menuApplication);

		// ------ menu Aide ------
		JMenu menuAide = new JMenu("Aide");
		menuAide.setMnemonic(KeyEvent.VK_M);
		actionAfficherCommentFaire = new ActionAfficherCommentFaire();
		JMenuItem itemCommentFaire = new JMenuItem(actionAfficherCommentFaire);
		menuAide.add(itemCommentFaire);
		actionAfficherConfig = new ActionAfficherConfig();
		JMenuItem itemConfig = new JMenuItem(actionAfficherConfig);
		menuAide.add(itemConfig);
		barre.add(menuAide);
		// ****** Fin barre de menu ******

		// ****** Création des fenêtres ******
		// ------ fenêtre conversion ------
		conversion = new FenetreConversion(actionAfficherConversion);
		this.add(conversion);
		// ------ fenêtre texte ------
		texte = new FenetreTexte(actionAfficherTexte);
		this.add(texte);
		// ------ fenêtre diaporama ------
		diaporama = new FenetreDiaporama(actionAfficherDiaporama);
		this.add(diaporama);
		// ------ fenêtre boutons ------
		boutons = new FenetreBoutons(this, actionAfficherBoutons);
		this.add(boutons);
		// ------ fenêtre commentFaire ------
		commentFaire = new FenetreCommentFaire(actionAfficherCommentFaire);
		this.add(commentFaire);
		// ------ fenêtre configuration ------
		config = new FenetreConfig(actionAfficherConfig, utilisateur);
		this.add(config);
		// ****** Fin création fenêtres ******
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(600, 300);
		this.setLocationRelativeTo(null);

		Application.popSuggestion(utilisateur);
		setVisible(true);
	}

	/**
	 * Ajoute un utilisateur s'il n'existe pas déjà, et initialise ses options de
	 * configuration par défaut.
	 * 
	 * @param utilisateur
	 */
	public static void ajouterUtilisateur(String utilisateur) {

		File f = new File(utilisateur + ".xml");
		if (!f.exists()) {
			OutputStream os = null;
			Properties prop = new Properties();
			prop.setProperty("conversion", "auto");
			prop.setProperty("saisie", "auto");
			prop.setProperty("diaporama", "auto");
			prop.setProperty("boutons", "auto");

			prop.setProperty("key_conversion", "on");
			prop.setProperty("key_saisie", "on");
			prop.setProperty("key_diaporama", "on");
			prop.setProperty("key_boutons", "on");

			prop.setProperty("info_conversion", "on");
			prop.setProperty("info_saisie", "on");
			prop.setProperty("info_diaporama", "on");
			prop.setProperty("info_boutons", "on");
			try {
				os = new FileOutputStream(utilisateur + ".xml");
				prop.storeToXML(os, null);
				os.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Lit l'option de configuration passée en paramètre appartenant à l'utilisateur
	 * passé en paramètre également.
	 * 
	 * @param utilisateur
	 * @param donnee
	 */
	public static String lireConfig(String utilisateur, String donnee) {

		try {
			Properties prop = new Properties();
			prop.loadFromXML(new FileInputStream(utilisateur + ".xml"));
			return prop.getProperty(donnee);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Modifie l'option de configuration passée en paramètre appartenant à
	 * l'utilisateur passé en paramètre également.
	 * 
	 * @param utilisateur
	 * @param donnee
	 * @param valeur
	 */
	public static void modifierConfig(String utilisateur, String donnee, String valeur) {

		try {

			Properties oldProp = new Properties();
			oldProp.loadFromXML(new FileInputStream(utilisateur + ".xml"));

			Properties prop = new Properties();

			prop.setProperty("conversion", oldProp.getProperty("conversion"));
			prop.setProperty("saisie", oldProp.getProperty("saisie"));
			prop.setProperty("diaporama", oldProp.getProperty("diaporama"));
			prop.setProperty("boutons", oldProp.getProperty("boutons"));

			prop.setProperty("key_conversion", oldProp.getProperty("key_conversion"));
			prop.setProperty("key_saisie", oldProp.getProperty("key_saisie"));
			prop.setProperty("key_diaporama", oldProp.getProperty("key_diaporama"));
			prop.setProperty("key_boutons", oldProp.getProperty("key_boutons"));

			prop.setProperty("info_conversion", oldProp.getProperty("info_conversion"));
			prop.setProperty("info_saisie", oldProp.getProperty("info_saisie"));
			prop.setProperty("info_diaporama", oldProp.getProperty("info_diaporama"));
			prop.setProperty("info_boutons", oldProp.getProperty("info_boutons"));

			switch(oldProp.getProperty("conversion")){
				case "auto" : itemConversion.setEnabled(true); break;
				case "on" : itemConversion.setEnabled(true); break;
				case "off" : itemConversion.setEnabled(false); break;
			}

			switch(oldProp.getProperty("saisie")){
				case "auto" : itemTexte.setEnabled(true); break;
				case "on" : itemTexte.setEnabled(true); break;
				case "off" : itemTexte.setEnabled(false); break;
			}

			switch(oldProp.getProperty("diaporama")){
				case "auto" : itemDiaporama.setEnabled(true); break;
				case "on" : itemDiaporama.setEnabled(true); break;
				case "off" : itemDiaporama.setEnabled(false); break;
			}

			switch(oldProp.getProperty("boutons")){
				case "auto" : itemBoutons.setEnabled(true); break;
				case "on" : itemBoutons.setEnabled(true); break;
				case "off" : itemBoutons.setEnabled(false); break;
			}

			prop.setProperty(donnee, valeur);

			OutputStream os = new FileOutputStream(utilisateur + ".xml");
			prop.storeToXML(os, null);
			os.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Lit l'option de configuration passée en paramètre appartenant à l'utilisateur
	 * passé en paramètre également.
	 * 
	 * @param utilisateur
	 * @param donnee
	 */
	public static void popSuggestion(String utilisateur) {

		try {
			ArrayList<String> alSuggestion = new ArrayList<String>();
			Properties prop = new Properties();
			prop.loadFromXML(new FileInputStream(utilisateur + ".xml"));
			if (prop.getProperty("key_conversion").equals("on"))
				alSuggestion.add("key_conversion");
			if (prop.getProperty("key_saisie").equals("on"))
				alSuggestion.add("key_saisie");
			if (prop.getProperty("key_diaporama").equals("on"))
				alSuggestion.add("key_diaporama");
			if (prop.getProperty("key_boutons").equals("on"))
				alSuggestion.add("key_boutons");
			if (prop.getProperty("info_conversion").equals("on"))
				alSuggestion.add("info_conversion");
			if (prop.getProperty("info_saisie").equals("on"))
				alSuggestion.add("info_saisie");
			if (prop.getProperty("info_diaporama").equals("on"))
				alSuggestion.add("info_diaporama");
			if (prop.getProperty("info_boutons").equals("on"))
				alSuggestion.add("info_boutons");

			int rand = (int) (Math.random() * alSuggestion.size());
			new FenetreSuggestion(alSuggestion.get(rand), utilisateur);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private class ActionAfficherBoutons extends AbstractAction {
		public ActionAfficherBoutons() {
			super("Boutons");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_B);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			boutons.setVisible(true);
			enableBoutons(false);
		}
	}

	private class ActionAfficherDiaporama extends AbstractAction {
		public ActionAfficherDiaporama() {
			super("Diaporama");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_D);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			diaporama.setVisible(true);
			enableDiaporama(false);
		}
	}

	private class ActionAfficherTexte extends AbstractAction {
		public ActionAfficherTexte() {
			super("Saisie de texte");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_T);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			texte.setVisible(true);
			enableTexte(false);
		}
	}

	public class ActionAfficherConversion extends AbstractAction {
		public ActionAfficherConversion() {
			super("Conversion Celsius/Farenheit");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_C);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			conversion.setVisible(true);
			enableConversion(false);
		}
	}

	public class ActionAfficherCommentFaire extends AbstractAction {
		public ActionAfficherCommentFaire() {
			super("Comment faire ?");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_U);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			commentFaire.setVisible(true);
			enableCommentFaire(false);
		}
	}

	public class ActionAfficherConfig extends AbstractAction {
		public ActionAfficherConfig() {
			super("Configuration des menus");
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_J, InputEvent.CTRL_DOWN_MASK));
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_J);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			config.setVisible(true);
			enableConfig(false);
		}
	}

	public void enableConversion(boolean b) {
		actionAfficherConversion.setEnabled(b);
	}

	public void enableTexte(boolean b) {
		actionAfficherTexte.setEnabled(b);
	}

	public void enableDiaporama(boolean b) {
		actionAfficherDiaporama.setEnabled(b);
	}

	public void enableBoutons(boolean b) {
		actionAfficherBoutons.setEnabled(b);
	}

	public void enableCommentFaire(boolean b) {
		actionAfficherCommentFaire.setEnabled(b);
	}

	public void enableConfig(boolean b) {
		actionAfficherConfig.setEnabled(b);
	}

	public Action getActionAfficherConversion() {
		return actionAfficherConversion;
	}

	public Action getActionAfficherTexte() {
		return actionAfficherTexte;
	}

	public Action getActionAfficherDiaporama() {
		return actionAfficherDiaporama;
	}

	public static void main(String[] args) {
		if (args.length == 1)
			new Application(args[0]);
		else
			new Application("default");
	}

}
